@@ -11,7 +11,7 @@
 #include <QAnimationDriver>
 
 #include <gst/gl/gl.h>
-#include "gstqtgl.h"
+#include <gst/gl/gstglfuncs.h>
 
 #include "qtglrenderer.h"
 #include "gstqtglutility.h"
@@ -22,7 +22,7 @@ GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 static void
 init_debug (void)
 {
-  static gsize _debug;
+  static volatile gsize _debug;
 
   if (g_once_init_enter (&_debug)) {
     GST_DEBUG_CATEGORY_INIT (GST_CAT_DEFAULT, "qtglrenderer", 0,
@@ -115,7 +115,7 @@ typedef enum
 
 struct SharedRenderData
 {
-  int refcount;
+  volatile int refcount;
   SharedRenderDataState state;
   GMutex lock;
   GCond cond;
@@ -130,7 +130,7 @@ shared_render_data_new (void)
 {
   struct SharedRenderData *ret = g_new0 (struct SharedRenderData, 1);
 
-  g_atomic_int_set (&ret->refcount, 1);
+  ret->refcount = 1;
   g_mutex_init (&ret->lock);
 
   return ret;
@@ -152,7 +152,7 @@ shared_render_data_free (struct SharedRenderData * data)
     delete data->m_context;
   data->m_context = nullptr;
   if (data->m_surface)
-    data->m_surface->deleteLater();
+    delete data->m_surface;
   data->m_surface = nullptr;
 }
 
@@ -281,6 +281,7 @@ bool CreateSurfaceWorker::event(QEvent * ev)
         g_mutex_lock (&m_sharedRenderData->lock);
         m_sharedRenderData->m_surface = new GstBackingSurface;
         m_sharedRenderData->m_surface->create();
+        m_sharedRenderData->m_surface->moveToThread (m_sharedRenderData->m_renderThread);
         GST_TRACE ("%p created surface %p", m_sharedRenderData,
             m_sharedRenderData->m_surface);
         g_cond_broadcast (&m_sharedRenderData->cond);
@@ -503,11 +504,6 @@ void GstQuickRenderer::stopGL ()
     if (m_sharedRenderData)
         shared_render_data_unref (m_sharedRenderData);
     m_sharedRenderData = NULL;
-
-    /* XXX: reset the OpenGL context and drawable as Qt may have clobbered it.
-     * Fixes any attempt to access OpenGL after shutting down qmlgloverlay. */
-    gst_gl_context_activate (gl_context, FALSE);
-    gst_gl_context_activate (gl_context, TRUE);
 }
 
 void GstQuickRenderer::cleanup()
@@ -633,18 +629,6 @@ void GstQuickRenderer::initializeGstGL ()
         return;
     }
     GST_INFO ("current QOpenGLContext %p", QOpenGLContext::currentContext());
-
-    /* XXX: Avoid an assertion inside QSGDefaultRenderContext::initialize()
-     * from an unused (in this scenario) property when using multiple
-     * QQuickRenderControl's with the same QOpenGLContext.
-     *
-     * First noticed with Qt 5.15.  Idea from:
-     * https://forum.qt.io/topic/55888/is-it-impossible-that-2-qquickrendercontrol-use-same-qopenglcontext/2
-     *
-     * ASSERT: "!m_gl->property(QSG_RENDERCONTEXT_PROPERTY).isValid()" in file /path/to/qt5/qtdeclarative/src/quick/scenegraph/qsgdefaultrendercontext.cpp, line 121
-     */
-    m_sharedRenderData->m_context->setProperty("_q_sgrendercontext", QVariant());
-
     m_renderControl->initialize(m_sharedRenderData->m_context);
 
     /* 1. QAnimationDriver's are thread-specific
