@@ -21,29 +21,28 @@
  */
 /**
  * SECTION:element-vp8enc
+ * @title: vp8enc
  * @see_also: vp8dec, webmmux, oggmux
  *
  * This element encodes raw video into a VP8 stream.
- * <ulink url="http://www.webmproject.org">VP8</ulink> is a royalty-free
- * video codec maintained by <ulink url="http://www.google.com/">Google
- * </ulink>. It's the successor of On2 VP3, which was the base of the
- * Theora video codec.
+ * [VP8](http://www.webmproject.org) is a royalty-free video codec maintained by
+ * [Google](http://www.google.com/). It's the successor of On2 VP3, which was
+ * the base of the Theora video codec.
  *
- * To control the quality of the encoding, the #GstVP8Enc::target-bitrate,
- * #GstVP8Enc::min-quantizer, #GstVP8Enc::max-quantizer or #GstVP8Enc::cq-level
+ * To control the quality of the encoding, the #GstVP8Enc:target-bitrate,
+ * #GstVP8Enc:min-quantizer, #GstVP8Enc:max-quantizer or #GstVP8Enc:cq-level
  * properties can be used. Which one is used depends on the mode selected by
- * the #GstVP8Enc::end-usage property.
- * See <ulink url="http://www.webmproject.org/docs/encoder-parameters/">Encoder Parameters</ulink>
+ * the #GstVP8Enc:end-usage property.
+ * See [Encoder Parameters](http://www.webmproject.org/docs/encoder-parameters/)
  * for explanation, examples for useful encoding parameters and more details
  * on the encoding parameters.
  *
- * <refsect2>
- * <title>Example pipeline</title>
+ * ## Example pipeline
  * |[
  * gst-launch-1.0 -v videotestsrc num-buffers=1000 ! vp8enc ! webmmux ! filesink location=videotestsrc.webm
  * ]| This example pipeline will encode a test video source to VP8 muxed in an
  * WebM container.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -141,10 +140,10 @@ gst_vp8_enc_class_init (GstVP8EncClass * klass)
   vpx_encoder_class = GST_VPX_ENC_CLASS (klass);
 
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_vp8_enc_src_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_vp8_enc_sink_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_vp8_enc_src_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_vp8_enc_sink_template);
 
   gst_element_class_set_static_metadata (element_class,
       "On2 VP8 Encoder",
@@ -289,7 +288,14 @@ gst_vp8_enc_process_frame_user_data (GstVPXEnc * enc,
     GstVideoCodecFrame * frame)
 {
   GstVP8EncUserData *user_data;
+
   user_data = gst_video_codec_frame_get_user_data (frame);
+
+  if (!user_data) {
+    GST_ERROR_OBJECT (enc, "Have no frame user data");
+    return NULL;
+  }
+
   if (user_data->image)
     g_slice_free (vpx_image_t, user_data->image);
   user_data->image = NULL;
@@ -301,7 +307,14 @@ gst_vp8_enc_handle_invisible_frame_buffer (GstVPXEnc * enc, void *user_data,
     GstBuffer * buffer)
 {
   GstVP8EncUserData *vp8_user_data = (GstVP8EncUserData *) user_data;
+
+  if (!vp8_user_data) {
+    GST_ERROR_OBJECT (enc, "Have no frame user data");
+    return GST_FLOW_ERROR;
+  }
+
   vp8_user_data->invisible = g_list_append (vp8_user_data->invisible, buffer);
+
   return GST_FLOW_OK;
 }
 
