@@ -47,7 +47,7 @@ GST_DEBUG_CATEGORY_STATIC (gst_vpxenc_debug);
 #define DEFAULT_PROFILE 0
 
 #define DEFAULT_RC_END_USAGE VPX_VBR
-#define DEFAULT_RC_TARGET_BITRATE 256000
+#define DEFAULT_RC_TARGET_BITRATE 0
 #define DEFAULT_RC_MIN_QUANTIZER 4
 #define DEFAULT_RC_MAX_QUANTIZER 63
 
@@ -88,7 +88,10 @@ GST_DEBUG_CATEGORY_STATIC (gst_vpxenc_debug);
 #define DEFAULT_DEADLINE VPX_DL_BEST_QUALITY
 #define DEFAULT_NOISE_SENSITIVITY 0
 #define DEFAULT_SHARPNESS 0
-#define DEFAULT_STATIC_THRESHOLD 0
+
+/* Use same default value as Chromium/webrtc. */
+#define DEFAULT_STATIC_THRESHOLD 1
+
 #define DEFAULT_TOKEN_PARTITIONS 0
 #define DEFAULT_ARNR_MAXFRAMES 0
 #define DEFAULT_ARNR_STRENGTH 3
@@ -99,6 +102,8 @@ GST_DEBUG_CATEGORY_STATIC (gst_vpxenc_debug);
 #define DEFAULT_TIMEBASE_N 0
 #define DEFAULT_TIMEBASE_D 1
 
+#define DEFAULT_BITS_PER_PIXEL 0.0434
+
 enum
 {
   PROP_0,
@@ -145,7 +150,8 @@ enum
   PROP_TUNING,
   PROP_CQ_LEVEL,
   PROP_MAX_INTRA_BITRATE_PCT,
-  PROP_TIMEBASE
+  PROP_TIMEBASE,
+  PROP_BITS_PER_PIXEL
 };
 
 
@@ -365,7 +371,8 @@ gst_vpx_enc_class_init (GstVPXEncClass * klass)
 
   g_object_class_install_property (gobject_class, PROP_RC_TARGET_BITRATE,
       g_param_spec_int ("target-bitrate", "Target bitrate",
-          "Target bitrate (in bits/sec)",
+          "Target bitrate (in bits/sec) (0: auto - bitrate depends on "
+          "resolution, see \"bits-per-pixel\" property for more info)",
           0, G_MAXINT, DEFAULT_RC_TARGET_BITRATE,
           (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
@@ -582,8 +589,8 @@ gst_vpx_enc_class_init (GstVPXEncClass * klass)
 
   g_object_class_install_property (gobject_class, PROP_STATIC_THRESHOLD,
       g_param_spec_int ("static-threshold", "Static Threshold",
-          "Motion detection threshold",
-          0, G_MAXINT, DEFAULT_STATIC_THRESHOLD,
+          "Motion detection threshold. Recommendation is to set 100 for "
+          "screen/window sharing", 0, G_MAXINT, DEFAULT_STATIC_THRESHOLD,
           (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
   g_object_class_install_property (gobject_class, PROP_TOKEN_PARTITIONS,
@@ -635,7 +642,22 @@ gst_vpx_enc_class_init (GstVPXEncClass * klass)
           0, 1, G_MAXINT, 1, DEFAULT_TIMEBASE_N, DEFAULT_TIMEBASE_D,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+  g_object_class_install_property (gobject_class, PROP_BITS_PER_PIXEL,
+      g_param_spec_float ("bits-per-pixel", "Bits per pixel",
+          "Factor to convert number of pixels to bitrate value "
+          "(only has an effect if target-bitrate=0)",
+          0.0, G_MAXFLOAT, DEFAULT_BITS_PER_PIXEL,
+          (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
+
   GST_DEBUG_CATEGORY_INIT (gst_vpxenc_debug, "vpxenc", 0, "VPX Encoder");
+
+  gst_type_mark_as_plugin_api (GST_VPX_ENC_END_USAGE_TYPE, 0);
+  gst_type_mark_as_plugin_api (GST_VPX_ENC_MULTIPASS_MODE_TYPE, 0);
+  gst_type_mark_as_plugin_api (GST_VPX_ENC_KF_MODE_TYPE, 0);
+  gst_type_mark_as_plugin_api (GST_VPX_ENC_TUNING_TYPE, 0);
+  gst_type_mark_as_plugin_api (GST_VPX_ENC_SCALING_MODE_TYPE, 0);
+  gst_type_mark_as_plugin_api (GST_VPX_ENC_TOKEN_PARTITIONS_TYPE, 0);
+  gst_type_mark_as_plugin_api (GST_VPX_ENC_ER_FLAGS_TYPE, 0);
 }
 
 static void
@@ -646,7 +668,7 @@ gst_vpx_enc_init (GstVPXEnc * gst_vpx_enc)
 
   gst_vpx_enc->cfg.rc_end_usage = DEFAULT_RC_END_USAGE;
   gst_vpx_enc->cfg.rc_target_bitrate = DEFAULT_RC_TARGET_BITRATE / 1000;
-  gst_vpx_enc->rc_target_bitrate_set = FALSE;
+  gst_vpx_enc->rc_target_bitrate_auto = DEFAULT_RC_TARGET_BITRATE == 0;
   gst_vpx_enc->cfg.rc_min_quantizer = DEFAULT_RC_MIN_QUANTIZER;
   gst_vpx_enc->cfg.rc_max_quantizer = DEFAULT_RC_MAX_QUANTIZER;
   gst_vpx_enc->cfg.rc_dropframe_thresh = DEFAULT_RC_DROPFRAME_THRESH;
@@ -694,6 +716,7 @@ gst_vpx_enc_init (GstVPXEnc * gst_vpx_enc)
   gst_vpx_enc->max_intra_bitrate_pct = DEFAULT_MAX_INTRA_BITRATE_PCT;
   gst_vpx_enc->timebase_n = DEFAULT_TIMEBASE_N;
   gst_vpx_enc->timebase_d = DEFAULT_TIMEBASE_D;
+  gst_vpx_enc->bits_per_pixel = DEFAULT_BITS_PER_PIXEL;
 
   gst_vpx_enc->cfg.g_profile = DEFAULT_PROFILE;
 
@@ -723,6 +746,42 @@ gst_vpx_enc_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+gst_vpx_enc_set_auto_bitrate (GstVPXEnc * encoder)
+{
+  if (encoder->input_state != NULL) {
+    guint size;
+    guint pixels_per_sec;
+    guint target_bitrate;
+    guint fps_n, fps_d;
+
+    if (GST_VIDEO_INFO_FPS_D (&encoder->input_state->info) != 0) {
+      fps_n = GST_VIDEO_INFO_FPS_N (&encoder->input_state->info);
+      fps_d = GST_VIDEO_INFO_FPS_D (&encoder->input_state->info);
+    } else {
+      /* otherwise assume 30 frames per second as a fallback */
+      fps_n = 30;
+      fps_d = 1;
+    }
+
+    size =
+        GST_VIDEO_INFO_WIDTH (&encoder->input_state->info) *
+        GST_VIDEO_INFO_HEIGHT (&encoder->input_state->info);
+    pixels_per_sec = size * fps_n / fps_d;
+    target_bitrate = pixels_per_sec * encoder->bits_per_pixel;
+
+    GST_DEBUG_OBJECT (encoder,
+        "Setting autobitrate for %ux%ux @ %u/%ufps %.4f = %ubps",
+        GST_VIDEO_INFO_WIDTH (&encoder->input_state->info),
+        GST_VIDEO_INFO_HEIGHT (&encoder->input_state->info),
+        GST_VIDEO_INFO_FPS_N (&encoder->input_state->info),
+        GST_VIDEO_INFO_FPS_D (&encoder->input_state->info),
+        encoder->bits_per_pixel, target_bitrate);
+
+    encoder->cfg.rc_target_bitrate = target_bitrate / 1000;
+  }
+}
+
 static void
 gst_vpx_enc_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec)
@@ -742,8 +801,13 @@ gst_vpx_enc_set_property (GObject * object, guint prop_id,
       global = TRUE;
       break;
     case PROP_RC_TARGET_BITRATE:
-      gst_vpx_enc->cfg.rc_target_bitrate = g_value_get_int (value) / 1000;
-      gst_vpx_enc->rc_target_bitrate_set = TRUE;
+      if (g_value_get_int (value) == 0) {
+        gst_vpx_enc_set_auto_bitrate (gst_vpx_enc);
+        gst_vpx_enc->rc_target_bitrate_auto = TRUE;
+      } else {
+        gst_vpx_enc->cfg.rc_target_bitrate = g_value_get_int (value) / 1000;
+        gst_vpx_enc->rc_target_bitrate_auto = FALSE;
+      }
       global = TRUE;
       break;
     case PROP_RC_MIN_QUANTIZER:
@@ -1086,6 +1150,13 @@ gst_vpx_enc_set_property (GObject * object, guint prop_id,
       gst_vpx_enc->timebase_n = gst_value_get_fraction_numerator (value);
       gst_vpx_enc->timebase_d = gst_value_get_fraction_denominator (value);
       break;
+    case PROP_BITS_PER_PIXEL:
+      gst_vpx_enc->bits_per_pixel = g_value_get_float (value);
+      if (gst_vpx_enc->rc_target_bitrate_auto) {
+        gst_vpx_enc_set_auto_bitrate (gst_vpx_enc);
+        global = TRUE;
+      }
+      break;
     default:
       break;
   }
@@ -1307,6 +1378,9 @@ gst_vpx_enc_get_property (GObject * object, guint prop_id, GValue * value,
       gst_value_set_fraction (value, gst_vpx_enc->timebase_n,
           gst_vpx_enc->timebase_d);
       break;
+    case PROP_BITS_PER_PIXEL:
+      g_value_set_float (value, gst_vpx_enc->bits_per_pixel);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -1441,14 +1515,6 @@ gst_vpx_enc_set_format (GstVideoEncoder * video_encoder,
   }
 
   encoder->cfg.g_profile = gst_vpx_enc_get_downstream_profile (encoder);
-
-  /* Scale default bitrate to our size */
-  if (!encoder->rc_target_bitrate_set)
-    encoder->cfg.rc_target_bitrate =
-        gst_util_uint64_scale (DEFAULT_RC_TARGET_BITRATE,
-        GST_VIDEO_INFO_WIDTH (info) * GST_VIDEO_INFO_HEIGHT (info),
-        320 * 240 * 1000);
-
   encoder->cfg.g_w = GST_VIDEO_INFO_WIDTH (info);
   encoder->cfg.g_h = GST_VIDEO_INFO_HEIGHT (info);
 
@@ -1630,6 +1696,10 @@ gst_vpx_enc_set_format (GstVideoEncoder * video_encoder,
     gst_video_codec_state_unref (encoder->input_state);
   encoder->input_state = gst_video_codec_state_ref (state);
 
+  /* Scale default bitrate to our size */
+  if (encoder->rc_target_bitrate_auto)
+    gst_vpx_enc_set_auto_bitrate (encoder);
+
   /* prepare cached image buffer setup */
   image = &encoder->image;
   memset (image, 0, sizeof (*image));
@@ -1720,7 +1790,8 @@ gst_vpx_enc_process (GstVPXEnc * encoder)
           encoder->cfg.g_timebase.den,
           encoder->cfg.g_timebase.num * (GstClockTime) GST_SECOND);
       GST_TRACE_OBJECT (encoder, "vpx pts: %" G_GINT64_FORMAT
-          ", gst frame pts: %" G_GINT64_FORMAT, pkt->data.frame.pts, pts);
+          ", gst frame pts: %" G_GINT64_FORMAT, (gint64) pkt->data.frame.pts,
+          (gint64) pts);
     } while (pkt->data.frame.pts > pts);
 
     g_assert (frame != NULL);
