@@ -95,6 +95,12 @@ G_BEGIN_DECLS
 #define FOURCC_avc1     GST_MAKE_FOURCC('a','v','c','1')
 #define FOURCC_avc3     GST_MAKE_FOURCC('a','v','c','3')
 #define FOURCC_avcC     GST_MAKE_FOURCC('a','v','c','C')
+#define FOURCC_c608     GST_MAKE_FOURCC('c','6','0','8')
+#define FOURCC_c708     GST_MAKE_FOURCC('c','7','0','8')
+#define FOURCC_ccdp     GST_MAKE_FOURCC('c','c','d','p')
+#define FOURCC_cdat     GST_MAKE_FOURCC('c','d','a','t')
+#define FOURCC_cdt2     GST_MAKE_FOURCC('c','d','t','2')
+#define FOURCC_clcp     GST_MAKE_FOURCC('c','l','c','p')
 #define FOURCC_clip     GST_MAKE_FOURCC('c','l','i','p')
 #define FOURCC_cmov     GST_MAKE_FOURCC('c','m','o','v')
 #define FOURCC_cmvd     GST_MAKE_FOURCC('c','m','v','d')
@@ -150,7 +156,11 @@ G_BEGIN_DECLS
 #define FOURCC_ilst     GST_MAKE_FOURCC('i','l','s','t')
 #define FOURCC_ima4     GST_MAKE_FOURCC('i','m','a','4')
 #define FOURCC_imap     GST_MAKE_FOURCC('i','m','a','p')
+#define FOURCC_s16l     GST_MAKE_FOURCC('s','1','6','l')
 #define FOURCC_in24     GST_MAKE_FOURCC('i','n','2','4')
+#define FOURCC_in32     GST_MAKE_FOURCC('i','n','3','2')
+#define FOURCC_fl64     GST_MAKE_FOURCC('f','l','6','4')
+#define FOURCC_fl32     GST_MAKE_FOURCC('f','l','3','2')
 #define FOURCC_jp2c     GST_MAKE_FOURCC('j','p','2','c')
 #define FOURCC_jpeg     GST_MAKE_FOURCC('j','p','e','g')
 #define FOURCC_keyw     GST_MAKE_FOURCC('k','e','y','w')
@@ -167,6 +177,7 @@ G_BEGIN_DECLS
 #define FOURCC_mhlr     GST_MAKE_FOURCC('m','h','l','r')
 #define FOURCC_minf     GST_MAKE_FOURCC('m','i','n','f')
 #define FOURCC_moov     GST_MAKE_FOURCC('m','o','o','v')
+#define FOURCC_mp3_     GST_MAKE_FOURCC('m','p','3',' ')
 #define FOURCC_mp4a     GST_MAKE_FOURCC('m','p','4','a')
 #define FOURCC_mp4s     GST_MAKE_FOURCC('m','p','4','s')
 #define FOURCC_mp4s     GST_MAKE_FOURCC('m','p','4','s')
@@ -174,6 +185,7 @@ G_BEGIN_DECLS
 #define FOURCC_name     GST_MAKE_FOURCC('n','a','m','e')
 #define FOURCC_nclc     GST_MAKE_FOURCC('n','c','l','c')
 #define FOURCC_nclx     GST_MAKE_FOURCC('n','c','l','x')
+#define FOURCC_nmhd     GST_MAKE_FOURCC('n','m','h','d')
 #define FOURCC_opus     GST_MAKE_FOURCC('O','p','u','s')
 #define FOURCC_dops     GST_MAKE_FOURCC('d','O','p','s')
 #define FOURCC_pasp     GST_MAKE_FOURCC('p','a','s','p')
@@ -186,6 +198,7 @@ G_BEGIN_DECLS
 #define FOURCC_fiel     GST_MAKE_FOURCC('f','i','e','l')
 #define FOURCC_pcst     GST_MAKE_FOURCC('p','c','s','t')
 #define FOURCC_pgap     GST_MAKE_FOURCC('p','g','a','p')
+#define FOURCC_png      GST_MAKE_FOURCC('p','n','g',' ')
 #define FOURCC_pnot     GST_MAKE_FOURCC('p','n','o','t')
 #define FOURCC_qt__     GST_MAKE_FOURCC('q','t',' ',' ')
 #define FOURCC_qtim     GST_MAKE_FOURCC('q','t','i','m')
@@ -204,6 +217,7 @@ G_BEGIN_DECLS
 #define FOURCC_sbtl     GST_MAKE_FOURCC('s','b','t','l')
 #define FOURCC_sdp_     GST_MAKE_FOURCC('s','d','p',' ')
 #define FOURCC_sidx     GST_MAKE_FOURCC('s','i','d','x')
+#define FOURCC_skip     GST_MAKE_FOURCC('s','k','i','p')
 #define FOURCC_smhd     GST_MAKE_FOURCC('s','m','h','d')
 #define FOURCC_soaa     GST_MAKE_FOURCC('s','o','a','a')
 #define FOURCC_soal     GST_MAKE_FOURCC('s','o','a','l')
@@ -249,10 +263,20 @@ G_BEGIN_DECLS
 #define FOURCC_vc_1     GST_MAKE_FOURCC('v','c','-','1')
 #define FOURCC_vide     GST_MAKE_FOURCC('v','i','d','e')
 #define FOURCC_vmhd     GST_MAKE_FOURCC('v','m','h','d')
+#define FOURCC_vp08     GST_MAKE_FOURCC('v','p','0','8')
+#define FOURCC_vp09     GST_MAKE_FOURCC('v','p','0','9')
+#define FOURCC_vpcC     GST_MAKE_FOURCC('v','p','c','C')
 #define FOURCC_xvid     GST_MAKE_FOURCC('x','v','i','d')
 #define FOURCC_wave     GST_MAKE_FOURCC('w','a','v','e')
 #define FOURCC_wide     GST_MAKE_FOURCC('w','i','d','e')
 #define FOURCC_zlib     GST_MAKE_FOURCC('z','l','i','b')
+#define FOURCC_lpcm     GST_MAKE_FOURCC('l','p','c','m')
+#define FOURCC_av01     GST_MAKE_FOURCC('a','v','0','1')
+#define FOURCC_av1C     GST_MAKE_FOURCC('a','v','1','C')
+#define FOURCC_av1f     GST_MAKE_FOURCC('a','v','1','f')
+#define FOURCC_av1m     GST_MAKE_FOURCC('a','v','1','m')
+#define FOURCC_av1s     GST_MAKE_FOURCC('a','v','1','s')
+#define FOURCC_av1M     GST_MAKE_FOURCC('a','v','1','M')
 
 #define FOURCC_cfhd     GST_MAKE_FOURCC('C','F','H','D')
 #define FOURCC_ap4x     GST_MAKE_FOURCC('a','p','4','x')
