@@ -19,18 +19,18 @@
 
 /**
  * SECTION:element-rtpL16depay
+ * @title: rtpL16depay
  * @see_also: rtpL16pay
  *
  * Extract raw audio from RTP packets according to RFC 3551.
  * For detailed information see: http://www.rfc-editor.org/rfc/rfc3551.txt
  *
- * <refsect2>
- * <title>Example pipeline</title>
+ * ## Example pipeline
  * |[
  * gst-launch-1.0 udpsrc caps='application/x-rtp, media=(string)audio, clock-rate=(int)44100, encoding-name=(string)L16, encoding-params=(string)1, channels=(int)1, payload=(int)96' ! rtpL16depay ! pulsesink
  * ]| This example pipeline will depayload an RTP raw audio stream. Refer to
  * the rtpL16pay example to create the RTP stream.
- * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -209,6 +209,7 @@ gst_rtp_L16_depay_setcaps (GstRTPBaseDepayload * depayload, GstCaps * caps)
             GST_STR_NULL (channel_order), channels));
     /* create default NONE layout */
     gst_rtp_channels_create_default (channels, info->position);
+    info->flags |= GST_AUDIO_FLAG_UNPOSITIONED;
   }
 
   srccaps = gst_audio_info_to_caps (info);
@@ -232,6 +233,7 @@ gst_rtp_L16_depay_process (GstRTPBaseDepayload * depayload, GstRTPBuffer * rtp)
   GstBuffer *outbuf;
   gint payload_len;
   gboolean marker;
+  GstAudioInfo *info;
 
   rtpL16depay = GST_RTP_L16_DEPAY (depayload);
 
@@ -251,15 +253,19 @@ gst_rtp_L16_depay_process (GstRTPBaseDepayload * depayload, GstRTPBuffer * rtp)
   }
 
   outbuf = gst_buffer_make_writable (outbuf);
+  info = &rtpL16depay->info;
+
+  if (payload_len % info->bpf != 0)
+    goto wrong_payload_size;
+
   if (rtpL16depay->order &&
       !gst_audio_buffer_reorder_channels (outbuf,
-          rtpL16depay->info.finfo->format, rtpL16depay->info.channels,
-          rtpL16depay->info.position, rtpL16depay->order->pos)) {
+          info->finfo->format, info->channels,
+          info->position, rtpL16depay->order->pos)) {
     goto reorder_failed;
   }
 
-  gst_rtp_drop_meta (GST_ELEMENT_CAST (rtpL16depay), outbuf,
-      g_quark_from_static_string (GST_META_TAG_AUDIO_STR));
+  gst_rtp_drop_non_audio_meta (rtpL16depay, outbuf);
 
   return outbuf;
 
@@ -270,6 +276,12 @@ empty_packet:
         ("Empty Payload."), (NULL));
     return NULL;
   }
+wrong_payload_size:
+  {
+    GST_ELEMENT_WARNING (rtpL16depay, STREAM, DECODE,
+        ("Wrong Payload Size."), (NULL));
+    return NULL;
+  }
 reorder_failed:
   {
     GST_ELEMENT_ERROR (rtpL16depay, STREAM, DECODE,
