@@ -20,15 +20,63 @@
  * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
+
+ /**
+ * SECTION:element-rtpfunnel
+ * @title: rtpfunnel
+ * @see_also: rtpbasepaylaoder, rtpsession
+ *
+ * RTP funnel is basically like a normal funnel with a few added
+ * functionalities to support bundling.
+ *
+ * Bundle is the concept of sending multiple streams in a single RTP session.
+ * These can be both audio and video streams, and several of both.
+ * One of the advantages with bundling is that you can get away with fewer
+ * ports for sending and receiving media. Also the RTCP traffic gets more
+ * compact if you can report on multiple streams in a single sender/receiver
+ * report.
+ *
+ * One of the reasons for a specialized RTP funnel is that some messages
+ * coming upstream want to find their way back to the right stream,
+ * and a normal funnel can't know which of its sinkpads it should send
+ * these messages to. The RTP funnel achieves this by keeping track of the
+ * SSRC of each stream on its sinkpad, and then uses the fact that upstream
+ * events are tagged inside rtpbin with the appropriate SSRC, so that upon
+ * receiving such an event, the RTP funnel can do a simple lookup for the
+ * right pad to forward the event to.
+ *
+ * A good example here is the KeyUnit event. If several video encoders are
+ * being bundled together using the RTP funnel, and one of the decoders on
+ * the receiving side asks for a KeyUnit, typically a RTCP PLI message will
+ * be sent from the receiver to the sender, and this will be transformed into
+ * a GstForceKeyUnit event inside GstRTPSession, and sent upstream. The
+ * RTP funnel can than make sure that this event hits the right encoder based
+ * on the SSRC embedded in the event.
+ *
+ * Another feature of the RTP funnel is that it will mux together TWCC
+ * (Transport-Wide Congestion Control) sequence-numbers. The point being that
+ * it should increment "transport-wide", meaning potentially several
+ * bundled streams. Note that not *all* streams being bundled needs to be
+ * affected by this. As an example Google WebRTC will use bundle with audio
+ * and video, but will only use TWCC sequence-numbers for the video-stream(s).
+ *
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#include <gst/rtp/gstrtpbuffer.h>
+
 #include "gstrtpfunnel.h"
 
 GST_DEBUG_CATEGORY_STATIC (gst_rtp_funnel_debug);
 #define GST_CAT_DEFAULT gst_rtp_funnel_debug
 
+#define TWCC_EXTMAP_STR "http://www.ietf.org/id/draft-holmer-rmcat-transport-wide-cc-extensions-01"
+
+/**************** GstRTPFunnelPad ****************/
+
 struct _GstRtpFunnelPadClass
 {
   GstPadClass class;
@@ -38,8 +86,23 @@ struct _GstRtpFunnelPad
 {
   GstPad pad;
   guint32 ssrc;
+  gboolean has_twcc;
 };
 
+G_DEFINE_TYPE (GstRtpFunnelPad, gst_rtp_funnel_pad, GST_TYPE_PAD);
+
+static void
+gst_rtp_funnel_pad_class_init (G_GNUC_UNUSED GstRtpFunnelPadClass * klass)
+{
+}
+
+static void
+gst_rtp_funnel_pad_init (G_GNUC_UNUSED GstRtpFunnelPad * pad)
+{
+}
+
+/**************** GstRTPFunnel ****************/
+
 enum
 {
   PROP_0,
@@ -48,20 +111,6 @@ enum
 
 #define DEFAULT_COMMON_TS_OFFSET -1
 
-G_DEFINE_TYPE (GstRtpFunnelPad, gst_rtp_funnel_pad, GST_TYPE_PAD);
-
-static void
-gst_rtp_funnel_pad_class_init (GstRtpFunnelPadClass * klass)
-{
-  (void) klass;
-}
-
-static void
-gst_rtp_funnel_pad_init (GstRtpFunnelPad * pad)
-{
-  (void) pad;
-}
-
 struct _GstRtpFunnelClass
 {
   GstElementClass class;
@@ -78,6 +127,10 @@ struct _GstRtpFunnel
   /* The last pad data was chained on */
   GstPad *current_pad;
 
+  guint8 twcc_ext_id;           /* the negotiated twcc extmap id */
+  guint16 twcc_seqnum;          /* our internal twcc seqnum */
+  guint twcc_pads;              /* numer of sinkpads with negotiated twcc */
+
   /* properties */
   gint common_ts_offset;
 };
@@ -102,7 +155,7 @@ static void
 gst_rtp_funnel_send_sticky (GstRtpFunnel * funnel, GstPad * pad)
 {
   GstEvent *stream_start;
-  GstEvent *caps;
+  GstEvent *caps_ev;
 
   if (!funnel->send_sticky_events)
     goto done;
@@ -113,8 +166,8 @@ gst_rtp_funnel_send_sticky (GstRtpFunnel * funnel, GstPad * pad)
     goto done;
   }
 
-  caps = gst_event_new_caps (funnel->srccaps);
-  if (caps && !gst_pad_push_event (funnel->srcpad, caps)) {
+  caps_ev = gst_event_new_caps (funnel->srccaps);
+  if (caps_ev && !gst_pad_push_event (funnel->srcpad, caps_ev)) {
     GST_ERROR_OBJECT (funnel, "Could not push caps");
     goto done;
   }
@@ -146,6 +199,43 @@ done:
   return;
 }
 
+static void
+gst_rtp_funnel_set_twcc_seqnum (GstRtpFunnel * funnel,
+    GstPad * pad, GstBuffer ** buf)
+{
+  GstRtpFunnelPad *fpad = GST_RTP_FUNNEL_PAD_CAST (pad);
+  GstRTPBuffer rtp = GST_RTP_BUFFER_INIT;
+
+  if (!funnel->twcc_ext_id || !fpad->has_twcc)
+    return;
+
+  *buf = gst_buffer_make_writable (*buf);
+
+  if (gst_rtp_buffer_map (*buf, GST_MAP_READWRITE, &rtp)) {
+    gpointer data;
+
+    /* if there already is a twcc-seqnum inside the packet */
+    if (gst_rtp_buffer_get_extension_onebyte_header (&rtp, funnel->twcc_ext_id,
+            0, &data, NULL)) {
+
+      /* with only one pad, we read the twcc-seqnum instead of writing it */
+      if (funnel->twcc_pads == 1) {
+        funnel->twcc_seqnum = GST_READ_UINT16_BE (data);
+      } else {
+        GST_WRITE_UINT16_BE (data, funnel->twcc_seqnum);
+      }
+    } else {
+      guint16 seq_be;
+      GST_WRITE_UINT16_BE (&seq_be, funnel->twcc_seqnum);
+      gst_rtp_buffer_add_extension_onebyte_header (&rtp, funnel->twcc_ext_id,
+          &seq_be, 2);
+    }
+  }
+  gst_rtp_buffer_unmap (&rtp);
+
+  funnel->twcc_seqnum++;
+}
+
 static GstFlowReturn
 gst_rtp_funnel_sink_chain_object (GstPad * pad, GstRtpFunnel * funnel,
     gboolean is_list, GstMiniObject * obj)
@@ -159,11 +249,13 @@ gst_rtp_funnel_sink_chain_object (GstPad * pad, GstRtpFunnel * funnel,
   gst_rtp_funnel_send_sticky (funnel, pad);
   gst_rtp_funnel_forward_segment (funnel, pad);
 
-  if (is_list)
+  if (is_list) {
     res = gst_pad_push_list (funnel->srcpad, GST_BUFFER_LIST_CAST (obj));
-  else
-    res = gst_pad_push (funnel->srcpad, GST_BUFFER_CAST (obj));
-
+  } else {
+    GstBuffer *buf = GST_BUFFER_CAST (obj);
+    gst_rtp_funnel_set_twcc_seqnum (funnel, pad, &buf);
+    res = gst_pad_push (funnel->srcpad, buf);
+  }
   GST_PAD_STREAM_UNLOCK (funnel->srcpad);
 
   return res;
@@ -188,10 +280,55 @@ gst_rtp_funnel_sink_chain (GstPad * pad, GstObject * parent, GstBuffer * buffer)
       GST_MINI_OBJECT_CAST (buffer));
 }
 
+static void
+gst_rtp_funnel_set_twcc_ext_id (GstRtpFunnel * funnel, guint8 twcc_ext_id)
+{
+  gchar *name;
+
+  if (funnel->twcc_ext_id == twcc_ext_id)
+    return;
+
+  name = g_strdup_printf ("extmap-%u", twcc_ext_id);
+  gst_caps_set_simple (funnel->srccaps, name, G_TYPE_STRING, TWCC_EXTMAP_STR,
+      NULL);
+  g_free (name);
+
+  /* make sure we update the sticky with the new caps */
+  funnel->send_sticky_events = TRUE;
+
+  GST_INFO_OBJECT (funnel, "Setting twcc-ext-id to %u", twcc_ext_id);
+  funnel->twcc_ext_id = twcc_ext_id;
+}
+
+static guint8
+_get_extmap_id_for_attribute (const GstStructure * s, const gchar * ext_name)
+{
+  guint i;
+  guint8 extmap_id = 0;
+  guint n_fields = gst_structure_n_fields (s);
+
+  for (i = 0; i < n_fields; i++) {
+    const gchar *field_name = gst_structure_nth_field_name (s, i);
+    if (g_str_has_prefix (field_name, "extmap-")) {
+      const gchar *str = gst_structure_get_string (s, field_name);
+      if (str && g_strcmp0 (str, ext_name) == 0) {
+        gint64 id = g_ascii_strtoll (field_name + 7, NULL, 10);
+        if (id > 0 && id < 15) {
+          extmap_id = id;
+          break;
+        }
+      }
+    }
+  }
+  return extmap_id;
+}
+
 static gboolean
 gst_rtp_funnel_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
 {
   GstRtpFunnel *funnel = GST_RTP_FUNNEL_CAST (parent);
+  GstRtpFunnelPad *fpad = GST_RTP_FUNNEL_PAD_CAST (pad);
+
   gboolean forward = TRUE;
   gboolean ret = TRUE;
 
@@ -207,6 +344,7 @@ gst_rtp_funnel_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
       GstCaps *caps;
       GstStructure *s;
       guint ssrc;
+      guint8 ext_id;
       gst_event_parse_caps (event, &caps);
 
       if (!gst_caps_can_intersect (funnel->srccaps, caps)) {
@@ -217,13 +355,18 @@ gst_rtp_funnel_sink_event (GstPad * pad, GstObject * parent, GstEvent * event)
 
       s = gst_caps_get_structure (caps, 0);
       if (gst_structure_get_uint (s, "ssrc", &ssrc)) {
-        GstRtpFunnelPad *fpad = GST_RTP_FUNNEL_PAD_CAST (pad);
         fpad->ssrc = ssrc;
         GST_DEBUG_OBJECT (pad, "Got ssrc: %u", ssrc);
         GST_OBJECT_LOCK (funnel);
         g_hash_table_insert (funnel->ssrc_to_pad, GUINT_TO_POINTER (ssrc), pad);
         GST_OBJECT_UNLOCK (funnel);
       }
+      ext_id = _get_extmap_id_for_attribute (s, TWCC_EXTMAP_STR);
+      if (ext_id > 0) {
+        fpad->has_twcc = TRUE;
+        funnel->twcc_pads++;
+        gst_rtp_funnel_set_twcc_ext_id (funnel, ext_id);
+      }
 
       forward = FALSE;
       break;
@@ -245,8 +388,7 @@ static gboolean
 gst_rtp_funnel_sink_query (GstPad * pad, GstObject * parent, GstQuery * query)
 {
   GstRtpFunnel *funnel = GST_RTP_FUNNEL_CAST (parent);
-  gboolean res = FALSE;
-  (void) funnel;
+  gboolean res = TRUE;
 
   switch (GST_QUERY_TYPE (query)) {
     case GST_QUERY_CAPS:
@@ -271,7 +413,20 @@ gst_rtp_funnel_sink_query (GstPad * pad, GstObject * parent, GstQuery * query)
       GST_DEBUG_OBJECT (pad, "Answering caps-query with caps: %"
           GST_PTR_FORMAT, new_caps);
       gst_caps_unref (new_caps);
-      res = TRUE;
+      break;
+    }
+    case GST_QUERY_ACCEPT_CAPS:
+    {
+      GstCaps *caps;
+      gboolean result;
+      gst_query_parse_accept_caps (query, &caps);
+      result = gst_caps_is_subset (caps, funnel->srccaps);
+      if (!result) {
+        GST_ERROR_OBJECT (pad,
+            "caps: %" GST_PTR_FORMAT " were not compatible with: %"
+            GST_PTR_FORMAT, caps, funnel->srccaps);
+      }
+      gst_query_set_accept_caps_result (query, result);
       break;
     }
     default:
