@@ -22,20 +22,13 @@
 #include <gst/base/gstbitreader.h>
 #include <gst/base/gstbitwriter.h>
 
-#include "gstrtputils.h"
-
 GST_DEBUG_CATEGORY_EXTERN (rtp_session_debug);
 #define GST_CAT_DEFAULT rtp_session_debug
 
-#define TWCC_EXTMAP_STR "http://www.ietf.org/id/draft-holmer-rmcat-transport-wide-cc-extensions-01"
-
 #define REF_TIME_UNIT (64 * GST_MSECOND)
 #define DELTA_UNIT (250 * GST_USECOND)
 #define MAX_TS_DELTA (0xff * DELTA_UNIT)
 
-#define STATUS_VECTOR_MAX_CAPACITY 14
-#define STATUS_VECTOR_TWO_BIT_MAX_CAPACITY 7
-
 typedef enum
 {
   RTP_TWCC_CHUNK_TYPE_RUN_LENGTH = 0,
@@ -67,7 +60,6 @@ typedef struct
   GstClockTime socket_ts;
   GstClockTime remote_ts;
   guint16 seqnum;
-  guint8 pt;
   guint size;
   gboolean lost;
 } SentPacket;
@@ -76,15 +68,11 @@ struct _RTPTWCCManager
 {
   GObject object;
 
-  guint8 send_ext_id;
-  guint8 recv_ext_id;
-  guint16 send_seqnum;
-
   guint mtu;
   guint max_packets_per_rtcp;
   GArray *recv_packets;
 
-  guint64 fb_pkt_count;
+  guint8 fb_pkt_count;
   gint32 last_seqnum;
 
   GArray *sent_packets;
@@ -95,17 +83,10 @@ struct _RTPTWCCManager
   guint64 recv_media_ssrc;
 
   guint16 expected_recv_seqnum;
-  guint16 packet_count_no_marker;
 
   gboolean first_fci_parse;
   guint16 expected_parsed_seqnum;
   guint8 expected_parsed_fb_pkt_count;
-
-  GstClockTime next_feedback_send_time;
-  GstClockTime feedback_interval;
-
-  guint64 remote_ts_base;
-  gint64 base_time_prev;
 };
 
 G_DEFINE_TYPE (RTPTWCCManager, rtp_twcc_manager, G_TYPE_OBJECT);
@@ -124,11 +105,6 @@ rtp_twcc_manager_init (RTPTWCCManager * twcc)
   twcc->recv_sender_ssrc = -1;
 
   twcc->first_fci_parse = TRUE;
-
-  twcc->feedback_interval = GST_CLOCK_TIME_NONE;
-  twcc->next_feedback_send_time = GST_CLOCK_TIME_NONE;
-
-  twcc->remote_ts_base = -1;
 }
 
 static void
@@ -166,35 +142,7 @@ recv_packet_init (RecvPacket * packet, guint16 seqnum, RTPPacketInfo * pinfo)
 {
   memset (packet, 0, sizeof (RecvPacket));
   packet->seqnum = seqnum;
-
-  if (GST_CLOCK_TIME_IS_VALID (pinfo->arrival_time))
-    packet->ts = pinfo->arrival_time;
-  else
-    packet->ts = pinfo->current_time;
-}
-
-void
-rtp_twcc_manager_parse_recv_ext_id (RTPTWCCManager * twcc,
-    const GstStructure * s)
-{
-  guint8 recv_ext_id = gst_rtp_get_extmap_id_for_attribute (s, TWCC_EXTMAP_STR);
-  if (recv_ext_id > 0) {
-    twcc->recv_ext_id = recv_ext_id;
-    GST_INFO ("TWCC enabled for recv using extension id: %u",
-        twcc->recv_ext_id);
-  }
-}
-
-void
-rtp_twcc_manager_parse_send_ext_id (RTPTWCCManager * twcc,
-    const GstStructure * s)
-{
-  guint8 send_ext_id = gst_rtp_get_extmap_id_for_attribute (s, TWCC_EXTMAP_STR);
-  if (send_ext_id > 0) {
-    twcc->send_ext_id = send_ext_id;
-    GST_INFO ("TWCC enabled for send using extension id: %u",
-        twcc->send_ext_id);
-  }
+  packet->ts = pinfo->running_time;
 }
 
 void
@@ -209,114 +157,6 @@ rtp_twcc_manager_set_mtu (RTPTWCCManager * twcc, guint mtu)
   twcc->max_packets_per_rtcp = ((twcc->mtu - 32) * 7) / (2 + 14);
 }
 
-void
-rtp_twcc_manager_set_feedback_interval (RTPTWCCManager * twcc,
-    GstClockTime feedback_interval)
-{
-  twcc->feedback_interval = feedback_interval;
-}
-
-GstClockTime
-rtp_twcc_manager_get_feedback_interval (RTPTWCCManager * twcc)
-{
-  return twcc->feedback_interval;
-}
-
-static gboolean
-_get_twcc_seqnum_data (RTPPacketInfo * pinfo, guint8 ext_id, gpointer * data)
-{
-  gboolean ret = FALSE;
-  guint size;
-
-  if (pinfo->header_ext &&
-      gst_rtp_buffer_get_extension_onebyte_header_from_bytes (pinfo->header_ext,
-          pinfo->header_ext_bit_pattern, ext_id, 0, data, &size)) {
-    if (size == 2)
-      ret = TRUE;
-  }
-  return ret;
-}
-
-static void
-sent_packet_init (SentPacket * packet, guint16 seqnum, RTPPacketInfo * pinfo,
-    GstRTPBuffer * rtp)
-{
-  packet->seqnum = seqnum;
-  packet->ts = pinfo->current_time;
-  packet->size = gst_rtp_buffer_get_payload_len (rtp);
-  packet->pt = gst_rtp_buffer_get_payload_type (rtp);
-  packet->remote_ts = GST_CLOCK_TIME_NONE;
-  packet->socket_ts = GST_CLOCK_TIME_NONE;
-  packet->lost = FALSE;
-}
-
-static void
-_set_twcc_seqnum_data (RTPTWCCManager * twcc, RTPPacketInfo * pinfo,
-    GstBuffer * buf, guint8 ext_id)
-{
-  SentPacket packet;
-  GstRTPBuffer rtp = GST_RTP_BUFFER_INIT;
-  gpointer data;
-
-  if (gst_rtp_buffer_map (buf, GST_MAP_READWRITE, &rtp)) {
-    if (gst_rtp_buffer_get_extension_onebyte_header (&rtp,
-            ext_id, 0, &data, NULL)) {
-      guint16 seqnum = twcc->send_seqnum++;
-
-      GST_WRITE_UINT16_BE (data, seqnum);
-      sent_packet_init (&packet, seqnum, pinfo, &rtp);
-      g_array_append_val (twcc->sent_packets, packet);
-
-      GST_LOG ("Send: twcc-seqnum: %u, pt: %u, marker: %d, len: %u, ts: %"
-          GST_TIME_FORMAT, seqnum, packet.pt, pinfo->marker, packet.size,
-          GST_TIME_ARGS (pinfo->current_time));
-    }
-    gst_rtp_buffer_unmap (&rtp);
-  }
-}
-
-static void
-rtp_twcc_manager_set_send_twcc_seqnum (RTPTWCCManager * twcc,
-    RTPPacketInfo * pinfo)
-{
-  if (GST_IS_BUFFER_LIST (pinfo->data)) {
-    GstBufferList *list;
-    guint i = 0;
-
-    pinfo->data = gst_buffer_list_make_writable (pinfo->data);
-
-    list = GST_BUFFER_LIST (pinfo->data);
-
-    for (i = 0; i < gst_buffer_list_length (list); i++) {
-      GstBuffer *buffer = gst_buffer_list_get_writable (list, i);
-
-      _set_twcc_seqnum_data (twcc, pinfo, buffer, twcc->send_ext_id);
-    }
-  } else {
-    pinfo->data = gst_buffer_make_writable (pinfo->data);
-    _set_twcc_seqnum_data (twcc, pinfo, pinfo->data, twcc->send_ext_id);
-  }
-}
-
-static gint32
-rtp_twcc_manager_get_recv_twcc_seqnum (RTPTWCCManager * twcc,
-    RTPPacketInfo * pinfo)
-{
-  gint32 val = -1;
-  gpointer data;
-
-  if (twcc->recv_ext_id == 0) {
-    GST_DEBUG ("Received TWCC packet, but no extension registered; ignoring");
-    return val;
-  }
-
-  if (_get_twcc_seqnum_data (pinfo, twcc->recv_ext_id, &data)) {
-    val = GST_READ_UINT16_BE (data);
-  }
-
-  return val;
-}
-
 static gint
 _twcc_seqnum_sort (gconstpointer a, gconstpointer b)
 {
@@ -357,7 +197,7 @@ rtp_twcc_write_run_length_chunk (GArray * packet_chunks,
     guint16 data = 0;
     guint len = MIN (run_length - written, 8191);
 
-    GST_LOG ("Writing a run-length of %u with status %u", len, status);
+    GST_LOG ("Writing a run-lenght of %u with status %u", len, status);
 
     gst_bit_writer_init_with_data (&writer, (guint8 *) & data, 2, FALSE);
     gst_bit_writer_put_bits_uint8 (&writer, RTP_TWCC_CHUNK_TYPE_RUN_LENGTH, 1);
@@ -417,7 +257,7 @@ chunk_bit_writer_get_available_slots (ChunkBitWriter * writer)
 static guint
 chunk_bit_writer_get_total_slots (ChunkBitWriter * writer)
 {
-  return STATUS_VECTOR_MAX_CAPACITY / writer->symbol_size;
+  return 14 / writer->symbol_size;
 }
 
 static void
@@ -500,43 +340,13 @@ run_lenght_helper_update (RunLengthHelper * rlh, RecvPacket * pkt)
   }
 }
 
-static guint
-_get_max_packets_capacity (guint symbol_size)
-{
-  if (symbol_size == 2)
-    return STATUS_VECTOR_TWO_BIT_MAX_CAPACITY;
-
-  return STATUS_VECTOR_MAX_CAPACITY;
-}
-
-static gboolean
-_pkt_fits_run_length_chunk (RecvPacket * pkt, guint packets_per_chunks,
-    guint remaining_packets)
-{
-  if (pkt->missing_run == 0) {
-    /* we have more or the same equal packets than the ones we can write in to a status chunk */
-    if (pkt->equal_run >= packets_per_chunks)
-      return TRUE;
-
-    /* we have more than one equal and not enough space for the remainings */
-    if (pkt->equal_run > 1 && remaining_packets > STATUS_VECTOR_MAX_CAPACITY)
-      return TRUE;
-
-    /* we have all equal packets for the remaining to write */
-    if (pkt->equal_run == remaining_packets)
-      return TRUE;
-  }
-
-  return FALSE;
-}
-
 static void
 rtp_twcc_write_chunks (GArray * packet_chunks,
     GArray * twcc_packets, guint symbol_size)
 {
   ChunkBitWriter writer;
   guint i;
-  guint packets_per_chunks = _get_max_packets_capacity (symbol_size);
+  guint bits_per_chunks = 7 * symbol_size;
 
   chunk_bit_writer_init (&writer, packet_chunks, symbol_size);
 
@@ -544,26 +354,21 @@ rtp_twcc_write_chunks (GArray * packet_chunks,
     RecvPacket *pkt = &g_array_index (twcc_packets, RecvPacket, i);
     guint remaining_packets = twcc_packets->len - i;
 
-    GST_LOG
-        ("About to write pkt: #%u missing_run: %u equal_run: %u status: %u, remaining_packets: %u",
-        pkt->seqnum, pkt->missing_run, pkt->equal_run, pkt->status,
-        remaining_packets);
-
     /* we can only start a run-length chunk if the status-chunk is
        completed */
     if (chunk_bit_writer_is_empty (&writer)) {
       /* first write in any preceeding gaps, we use run-length
          if it would take up more than one chunk (14/7) */
-      if (pkt->missing_run > packets_per_chunks) {
+      if (pkt->missing_run > bits_per_chunks) {
         rtp_twcc_write_run_length_chunk (packet_chunks,
             RTP_TWCC_PACKET_STATUS_NOT_RECV, pkt->missing_run);
       }
 
       /* we have a run of the same status, write a run-length chunk and skip
          to the next point */
-      if (_pkt_fits_run_length_chunk (pkt, packets_per_chunks,
-              remaining_packets)) {
-
+      if (pkt->missing_run == 0 &&
+          (pkt->equal_run > bits_per_chunks ||
+              pkt->equal_run == remaining_packets)) {
         rtp_twcc_write_run_length_chunk (packet_chunks,
             pkt->status, pkt->equal_run);
         i += pkt->equal_run - 1;
@@ -599,24 +404,9 @@ rtp_twcc_manager_add_fci (RTPTWCCManager * twcc, GstRTCPPacket * packet)
   guint symbol_size = 1;
   GstClockTimeDiff delta_ts;
   gint64 delta_ts_rounded;
-  guint8 fb_pkt_count;
 
   g_array_sort (twcc->recv_packets, _twcc_seqnum_sort);
 
-  /* Quick scan to remove duplicates */
-  prev = &g_array_index (twcc->recv_packets, RecvPacket, 0);
-  for (i = 1; i < twcc->recv_packets->len;) {
-    RecvPacket *cur = &g_array_index (twcc->recv_packets, RecvPacket, i);
-
-    if (prev->seqnum == cur->seqnum) {
-      GST_DEBUG ("Removing duplicate packet #%u", cur->seqnum);
-      g_array_remove_index (twcc->recv_packets, i);
-    } else {
-      prev = cur;
-      i += 1;
-    }
-  }
-
   /* get first and last packet */
   first = &g_array_index (twcc->recv_packets, RecvPacket, 0);
   last =
@@ -625,19 +415,19 @@ rtp_twcc_manager_add_fci (RTPTWCCManager * twcc, GstRTCPPacket * packet)
 
   packet_count = last->seqnum - first->seqnum + 1;
   base_time = first->ts / REF_TIME_UNIT;
-  fb_pkt_count = (guint8) (twcc->fb_pkt_count % G_MAXUINT8);
 
   GST_WRITE_UINT16_BE (header.base_seqnum, first->seqnum);
   GST_WRITE_UINT16_BE (header.packet_count, packet_count);
   GST_WRITE_UINT24_BE (header.base_time, base_time);
-  GST_WRITE_UINT8 (header.fb_pkt_count, fb_pkt_count);
+  GST_WRITE_UINT8 (header.fb_pkt_count, twcc->fb_pkt_count);
 
   base_time *= REF_TIME_UNIT;
   ts_rounded = base_time;
 
   GST_DEBUG ("Created TWCC feedback: base_seqnum: #%u, packet_count: %u, "
       "base_time %" GST_TIME_FORMAT " fb_pkt_count: %u",
-      first->seqnum, packet_count, GST_TIME_ARGS (base_time), fb_pkt_count);
+      first->seqnum, packet_count, GST_TIME_ARGS (base_time),
+      twcc->fb_pkt_count);
 
   twcc->fb_pkt_count++;
   twcc->expected_recv_seqnum = first->seqnum + packet_count;
@@ -737,7 +527,24 @@ rtp_twcc_manager_create_feedback (RTPTWCCManager * twcc)
 static gboolean
 _exceeds_max_packets (RTPTWCCManager * twcc, guint16 seqnum)
 {
-  if (twcc->recv_packets->len + 1 > twcc->max_packets_per_rtcp)
+  RecvPacket *first, *last;
+  guint16 packet_count;
+
+  if (twcc->recv_packets->len == 0)
+    return FALSE;
+
+  /* find the delta betwen first stored packet and this seqnum */
+  first = &g_array_index (twcc->recv_packets, RecvPacket, 0);
+  packet_count = seqnum - first->seqnum + 1;
+  if (packet_count > twcc->max_packets_per_rtcp)
+    return TRUE;
+
+  /* then find the delta between last stored packet and this seqnum */
+  last =
+      &g_array_index (twcc->recv_packets, RecvPacket,
+      twcc->recv_packets->len - 1);
+  packet_count = seqnum - (last->seqnum + 1);
+  if (packet_count > twcc->max_packets_per_rtcp)
     return TRUE;
 
   return FALSE;
@@ -752,41 +559,25 @@ _many_packets_some_lost (RTPTWCCManager * twcc, guint16 seqnum)
   RecvPacket *first;
   guint16 packet_count;
   guint received_packets = twcc->recv_packets->len;
-  guint lost_packets;
   if (received_packets == 0)
     return FALSE;
 
   first = &g_array_index (twcc->recv_packets, RecvPacket, 0);
   packet_count = seqnum - first->seqnum + 1;
-
-  /* If there are a high number of duplicates, we can't use the following
-   * metrics */
-  if (received_packets > packet_count)
-    return FALSE;
-
-  /* check if we lost half of the threshold */
-  lost_packets = packet_count - received_packets;
-  if (received_packets >= 30 && lost_packets >= 60)
-    return TRUE;
-
-  /* we have lost the marker bit for some and lost some */
-  if (twcc->packet_count_no_marker >= 10 && lost_packets >= 60)
+  /* packet-count larger than recevied-packets means we have lost packets */
+  if (packet_count >= 30 && packet_count > received_packets)
     return TRUE;
 
   return FALSE;
 }
 
 gboolean
-rtp_twcc_manager_recv_packet (RTPTWCCManager * twcc, RTPPacketInfo * pinfo)
+rtp_twcc_manager_recv_packet (RTPTWCCManager * twcc,
+    guint16 seqnum, RTPPacketInfo * pinfo)
 {
   gboolean send_feedback = FALSE;
   RecvPacket packet;
-  gint32 seqnum;
-  gint diff;
-
-  seqnum = rtp_twcc_manager_get_recv_twcc_seqnum (twcc, pinfo);
-  if (seqnum == -1)
-    return FALSE;
+  gint32 diff;
 
   /* if this packet would exceed the capacity of our MTU, we create a feedback
      with the current packets, and start over with this one */
@@ -804,8 +595,8 @@ rtp_twcc_manager_recv_packet (RTPTWCCManager * twcc, RTPPacketInfo * pinfo)
   /* check if we are reordered, and treat it as lost if we already sent
      a feedback msg with a higher seqnum. If the diff is huge, treat
      it as a restart of a stream */
-  diff = gst_rtp_buffer_compare_seqnum (twcc->expected_recv_seqnum, seqnum);
-  if (twcc->fb_pkt_count > 0 && diff < 0) {
+  diff = (gint32) seqnum - (gint32) twcc->expected_recv_seqnum;
+  if (twcc->fb_pkt_count > 0 && diff < 0 && diff > -1000) {
     GST_INFO ("Received out of order packet (%u after %u), treating as lost",
         seqnum, twcc->expected_recv_seqnum);
     return FALSE;
@@ -815,36 +606,12 @@ rtp_twcc_manager_recv_packet (RTPTWCCManager * twcc, RTPPacketInfo * pinfo)
   recv_packet_init (&packet, seqnum, pinfo);
   g_array_append_val (twcc->recv_packets, packet);
   twcc->last_seqnum = seqnum;
+  GST_LOG ("Receive: twcc-seqnum: %u, marker: %d, ts: %" GST_TIME_FORMAT,
+      seqnum, pinfo->marker, GST_TIME_ARGS (pinfo->running_time));
 
-  GST_LOG ("Receive: twcc-seqnum: %u, pt: %u, marker: %d, ts: %"
-      GST_TIME_FORMAT, seqnum, pinfo->pt, pinfo->marker,
-      GST_TIME_ARGS (pinfo->arrival_time));
-
-  if (!pinfo->marker)
-    twcc->packet_count_no_marker++;
-
-  /* are we sending on an interval, or based on marker bit */
-  if (GST_CLOCK_TIME_IS_VALID (twcc->feedback_interval)) {
-    if (!GST_CLOCK_TIME_IS_VALID (twcc->next_feedback_send_time))
-      twcc->next_feedback_send_time =
-          pinfo->running_time + twcc->feedback_interval;
-
-    if (pinfo->running_time >= twcc->next_feedback_send_time) {
-      GST_LOG ("Generating feedback : Exceeded feedback interval %"
-          GST_TIME_FORMAT, GST_TIME_ARGS (twcc->feedback_interval));
-      rtp_twcc_manager_create_feedback (twcc);
-      send_feedback = TRUE;
-
-      while (pinfo->running_time >= twcc->next_feedback_send_time)
-        twcc->next_feedback_send_time += twcc->feedback_interval;
-    }
-  } else if (pinfo->marker || _many_packets_some_lost (twcc, seqnum)) {
-    GST_LOG ("Generating feedback because of %s",
-        pinfo->marker ? "marker packet" : "many packets some lost");
+  if (pinfo->marker || _many_packets_some_lost (twcc, seqnum)) {
     rtp_twcc_manager_create_feedback (twcc);
     send_feedback = TRUE;
-
-    twcc->packet_count_no_marker = 0;
   }
 
   return send_feedback;
@@ -875,13 +642,39 @@ rtp_twcc_manager_get_feedback (RTPTWCCManager * twcc, guint sender_ssrc)
   return buf;
 }
 
-void
-rtp_twcc_manager_send_packet (RTPTWCCManager * twcc, RTPPacketInfo * pinfo)
+static void
+sent_packet_init (SentPacket * packet, guint16 seqnum, RTPPacketInfo * pinfo)
 {
-  if (twcc->send_ext_id == 0)
-    return;
+  packet->seqnum = seqnum;
+  packet->ts = pinfo->running_time;
+  packet->size = pinfo->payload_len;
+  packet->remote_ts = GST_CLOCK_TIME_NONE;
+  packet->socket_ts = GST_CLOCK_TIME_NONE;
+  packet->lost = FALSE;
+}
 
-  rtp_twcc_manager_set_send_twcc_seqnum (twcc, pinfo);
+void
+rtp_twcc_manager_send_packet (RTPTWCCManager * twcc,
+    guint16 seqnum, RTPPacketInfo * pinfo)
+{
+  SentPacket packet;
+  sent_packet_init (&packet, seqnum, pinfo);
+  g_array_append_val (twcc->sent_packets, packet);
+
+  GST_LOG ("Send: twcc-seqnum: %u, marker: %d, ts: %" GST_TIME_FORMAT,
+      seqnum, pinfo->marker, GST_TIME_ARGS (pinfo->running_time));
+}
+
+void
+rtp_twcc_manager_set_send_packet_ts (RTPTWCCManager * twcc,
+    guint packet_id, GstClockTime ts)
+{
+  SentPacket *pkt = NULL;
+  pkt = &g_array_index (twcc->sent_packets, SentPacket, packet_id);
+  if (pkt) {
+    pkt->socket_ts = ts;
+    GST_DEBUG ("assigning: pkt-id: %u to packet: %u", packet_id, pkt->seqnum);
+  }
 }
 
 static void
@@ -903,14 +696,14 @@ static guint
 _parse_run_length_chunk (GstBitReader * reader, GArray * twcc_packets,
     guint16 seqnum_offset, guint remaining_packets)
 {
-  guint16 run_length;
+  guint run_length;
   guint8 status_code;
   guint i;
 
   gst_bit_reader_get_bits_uint8 (reader, &status_code, 2);
-  gst_bit_reader_get_bits_uint16 (reader, &run_length, 13);
 
-  run_length = MIN (remaining_packets, run_length);
+  run_length = *(guint16 *) reader->data & ~0xE0;       /* mask out the 3 last bits */
+  run_length = MIN (remaining_packets, GST_READ_UINT16_BE (&run_length));
 
   for (i = 0; i < run_length; i++) {
     _add_twcc_packet (twcc_packets, seqnum_offset + i, status_code);
@@ -957,7 +750,7 @@ _prune_sent_packets (RTPTWCCManager * twcc, GArray * twcc_packets)
 
   last_idx = last->seqnum - first->seqnum;
 
-  if (last_idx < twcc->sent_packets->len)
+  if (last_idx >= twcc->sent_packets->len)
     g_array_remove_range (twcc->sent_packets, 0, last_idx);
 }
 
@@ -966,7 +759,6 @@ _check_for_lost_packets (RTPTWCCManager * twcc, GArray * twcc_packets,
     guint16 base_seqnum, guint16 packet_count, guint8 fb_pkt_count)
 {
   guint packets_lost;
-  gint8 fb_pkt_count_diff;
   guint i;
 
   /* first packet */
@@ -975,31 +767,22 @@ _check_for_lost_packets (RTPTWCCManager * twcc, GArray * twcc_packets,
     goto done;
   }
 
-  fb_pkt_count_diff =
-      (gint8) (fb_pkt_count - twcc->expected_parsed_fb_pkt_count);
-
   /* we have gone backwards, don't reset the expectations,
      but process the packet nonetheless */
-  if (fb_pkt_count_diff < 0) {
-    GST_DEBUG ("feedback packet count going backwards (%u < %u)",
+  if (fb_pkt_count < twcc->expected_parsed_fb_pkt_count) {
+    GST_WARNING ("feedback packet count going backwards (%u < %u)",
         fb_pkt_count, twcc->expected_parsed_fb_pkt_count);
     return;
   }
 
   /* we have jumped forwards, reset expectations, but don't trigger
      lost packets in case the missing fb-packet(s) arrive later */
-  if (fb_pkt_count_diff > 0) {
-    GST_DEBUG ("feedback packet count jumped ahead (%u > %u)",
+  if (fb_pkt_count > twcc->expected_parsed_fb_pkt_count) {
+    GST_WARNING ("feedback packet count jumped ahead (%u > %u)",
         fb_pkt_count, twcc->expected_parsed_fb_pkt_count);
     goto done;
   }
 
-  if (base_seqnum < twcc->expected_parsed_seqnum) {
-    GST_DEBUG ("twcc seqnum is older than expected  (%u < %u)", base_seqnum,
-        twcc->expected_parsed_seqnum);
-    return;
-  }
-
   packets_lost = base_seqnum - twcc->expected_parsed_seqnum;
   for (i = 0; i < packets_lost; i++) {
     _add_twcc_packet (twcc_packets, twcc->expected_parsed_seqnum + i,
@@ -1020,7 +803,6 @@ rtp_twcc_manager_parse_fci (RTPTWCCManager * twcc,
   guint16 base_seqnum;
   guint16 packet_count;
   GstClockTime base_time;
-  gint64 base_time_ext;
   GstClockTime ts_rounded;
   guint8 fb_pkt_count;
   guint packets_parsed = 0;
@@ -1035,17 +817,12 @@ rtp_twcc_manager_parse_fci (RTPTWCCManager * twcc,
 
   base_seqnum = GST_READ_UINT16_BE (&fci_data[0]);
   packet_count = GST_READ_UINT16_BE (&fci_data[2]);
-  base_time = GST_READ_UINT24_BE (&fci_data[4]);
-  /* Sign-extend the base_time from a 24-bit integer into a 64-bit signed integer
-   * so that we can calculate diffs with regular 64-bit operations. */
-  base_time_ext =
-      (base_time & 0x800000) ? base_time | 0xFFFFFFFFFF800000 : base_time;
+  base_time = GST_READ_UINT24_BE (&fci_data[4]) * REF_TIME_UNIT;
   fb_pkt_count = fci_data[7];
 
   GST_DEBUG ("Parsed TWCC feedback: base_seqnum: #%u, packet_count: %u, "
       "base_time %" GST_TIME_FORMAT " fb_pkt_count: %u",
-      base_seqnum, packet_count, GST_TIME_ARGS (base_time * REF_TIME_UNIT),
-      fb_pkt_count);
+      base_seqnum, packet_count, GST_TIME_ARGS (base_time), fb_pkt_count);
 
   twcc_packets = g_array_sized_new (FALSE, FALSE,
       sizeof (RTPTWCCPacket), packet_count);
@@ -1075,21 +852,7 @@ rtp_twcc_manager_parse_fci (RTPTWCCManager * twcc,
   if (twcc->sent_packets->len > 0)
     first_sent_pkt = &g_array_index (twcc->sent_packets, SentPacket, 0);
 
-  if (twcc->remote_ts_base == -1) {
-    /* Add an initial offset of 1 << 24 so that we don't risk going below 0 if
-     * a future extended timestamp is earlier than the first. */
-    twcc->remote_ts_base = (G_GINT64_CONSTANT (1) << 24) + base_time_ext;
-  } else {
-    /* Calculate our internal accumulated reference timestamp by continously
-     * adding the diff between the current and the previous sign-extended
-     * reference time. */
-    twcc->remote_ts_base += base_time_ext - twcc->base_time_prev;
-  }
-  twcc->base_time_prev = base_time_ext;
-  /* Our internal accumulated reference time is in units of 64ms, propagate as
-   * GstClockTime in ns. */
-  ts_rounded = twcc->remote_ts_base * REF_TIME_UNIT;
-
+  ts_rounded = base_time;
   for (i = 0; i < twcc_packets->len; i++) {
     RTPTWCCPacket *pkt = &g_array_index (twcc_packets, RTPTWCCPacket, i);
     gint16 delta = 0;
@@ -1133,7 +896,6 @@ rtp_twcc_manager_parse_fci (RTPTWCCManager * twcc,
           pkt->local_ts = found->ts;
         }
         pkt->size = found->size;
-        pkt->pt = found->pt;
 
         GST_LOG ("matching pkt: #%u with local_ts: %" GST_TIME_FORMAT
             " size: %u", pkt->seqnum, GST_TIME_ARGS (pkt->local_ts), pkt->size);
