@@ -25,6 +25,7 @@
 #include <gst/rtp/rtp.h>
 #include "rtpstats.h"
 
+typedef struct _RTPTWCCManager RTPTWCCManager;
 typedef struct _RTPTWCCPacket RTPTWCCPacket;
 typedef enum _RTPTWCCPacketStatus RTPTWCCPacketStatus;
 
@@ -49,25 +50,19 @@ struct _RTPTWCCPacket
   RTPTWCCPacketStatus status;
   guint16 seqnum;
   guint size;
-  guint8 pt;
 };
 
 RTPTWCCManager * rtp_twcc_manager_new (guint mtu);
 
-void rtp_twcc_manager_parse_recv_ext_id (RTPTWCCManager * twcc,
-    const GstStructure * s);
-void rtp_twcc_manager_parse_send_ext_id (RTPTWCCManager * twcc,
-    const GstStructure * s);
-
 void rtp_twcc_manager_set_mtu (RTPTWCCManager * twcc, guint mtu);
-void rtp_twcc_manager_set_feedback_interval (RTPTWCCManager * twcc,
-    GstClockTime feedback_interval);
-GstClockTime rtp_twcc_manager_get_feedback_interval (RTPTWCCManager * twcc);
 
 gboolean rtp_twcc_manager_recv_packet (RTPTWCCManager * twcc,
-    RTPPacketInfo * pinfo);
+    guint16 seqnum, RTPPacketInfo * pinfo);
+
 void rtp_twcc_manager_send_packet (RTPTWCCManager * twcc,
-    RTPPacketInfo * pinfo);
+    guint16 seqnum, RTPPacketInfo * pinfo);
+void rtp_twcc_manager_set_send_packet_ts (RTPTWCCManager * twcc,
+    guint packet_id, GstClockTime ts);
 
 GstBuffer * rtp_twcc_manager_get_feedback (RTPTWCCManager * twcc,
     guint32 sender_ssrc);
