@@ -551,9 +551,9 @@ gst_v4l2_buffer_pool_set_config (GstBufferPool * bpool, GstStructure * config)
         | GST_V4L2_ALLOCATOR_FLAG_DMABUF_CREATE_BUFS);
   }
 
-  if (min_buffers < GST_V4L2_MIN_BUFFERS) {
+  if (min_buffers < GST_V4L2_MIN_BUFFERS (obj)) {
     updated = TRUE;
-    min_buffers = GST_V4L2_MIN_BUFFERS;
+    min_buffers = GST_V4L2_MIN_BUFFERS (obj);
     GST_INFO_OBJECT (pool, "increasing minimum buffers to %u", min_buffers);
   }
 
@@ -599,7 +599,7 @@ gst_v4l2_buffer_pool_set_config (GstBufferPool * bpool, GstStructure * config)
 done:
   ret = GST_BUFFER_POOL_CLASS (parent_class)->set_config (bpool, config);
 
-  /* If anything was changed documentation recommand to return FALSE */
+  /* If anything was changed documentation recommend to return FALSE */
   return !updated && ret;
 
   /* ERRORS */
@@ -650,12 +650,14 @@ gst_v4l2_buffer_pool_streamon (GstV4l2BufferPool * pool)
     case GST_V4L2_IO_DMABUF:
     case GST_V4L2_IO_DMABUF_IMPORT:
       if (!V4L2_TYPE_IS_OUTPUT (pool->obj->type)) {
+        guint i;
+
         /* For captures, we need to enqueue buffers before we start streaming,
-         * so the driver don't underflow immediatly. As we have put then back
+         * so the driver don't underflow immediately. As we have put then back
          * into the base class queue, resurrect them, then releasing will queue
          * them back. */
-        while (gst_v4l2_buffer_pool_resurrect_buffer (pool) == GST_FLOW_OK)
-          continue;
+        for (i = 0; i < pool->num_allocated; i++)
+          gst_v4l2_buffer_pool_resurrect_buffer (pool);
       }
 
       if (obj->ioctl (pool->video_fd, VIDIOC_STREAMON, &obj->type) < 0)
@@ -764,7 +766,7 @@ gst_v4l2_buffer_pool_start (GstBufferPool * bpool)
           &max_buffers))
     goto wrong_config;
 
-  min_latency = MAX (GST_V4L2_MIN_BUFFERS, obj->min_buffers);
+  min_latency = MAX (GST_V4L2_MIN_BUFFERS (obj), obj->min_buffers);
 
   switch (obj->mode) {
     case GST_V4L2_IO_RW:
@@ -791,8 +793,9 @@ gst_v4l2_buffer_pool_start (GstBufferPool * bpool)
 
       count = gst_v4l2_allocator_start (pool->vallocator, min_buffers,
           V4L2_MEMORY_MMAP);
+      pool->num_allocated = count;
 
-      if (count < GST_V4L2_MIN_BUFFERS) {
+      if (count < GST_V4L2_MIN_BUFFERS (obj)) {
         min_buffers = count;
         goto no_buffers;
       }
@@ -800,7 +803,7 @@ gst_v4l2_buffer_pool_start (GstBufferPool * bpool)
       /* V4L2 buffer pool are often very limited in the amount of buffers it
        * can offer. The copy_threshold will workaround this limitation by
        * falling back to copy if the pipeline needed more buffers. This also
-       * prevent having to do REQBUFS(N)/REQBUFS(0) everytime configure is
+       * prevent having to do REQBUFS(N)/REQBUFS(0) every time configure is
        * called. */
       if (count != min_buffers || pool->enable_copy_threshold) {
         GST_WARNING_OBJECT (pool,
@@ -818,9 +821,10 @@ gst_v4l2_buffer_pool_start (GstBufferPool * bpool)
       can_allocate =
           GST_V4L2_ALLOCATOR_CAN_ALLOCATE (pool->vallocator, USERPTR);
 
-      GST_DEBUG_OBJECT (pool, "requesting %d USERPTR buffers", min_buffers);
+      GST_DEBUG_OBJECT (pool, "requesting %d USERPTR import slots",
+          max_buffers);
 
-      count = gst_v4l2_allocator_start (pool->vallocator, min_buffers,
+      count = gst_v4l2_allocator_start (pool->vallocator, max_buffers,
           V4L2_MEMORY_USERPTR);
 
       /* There is no rational to not get what we asked */
@@ -838,9 +842,9 @@ gst_v4l2_buffer_pool_start (GstBufferPool * bpool)
 
       can_allocate = GST_V4L2_ALLOCATOR_CAN_ALLOCATE (pool->vallocator, DMABUF);
 
-      GST_DEBUG_OBJECT (pool, "requesting %d DMABUF buffers", min_buffers);
+      GST_DEBUG_OBJECT (pool, "requesting %d DMABUF import slots", max_buffers);
 
-      count = gst_v4l2_allocator_start (pool->vallocator, min_buffers,
+      count = gst_v4l2_allocator_start (pool->vallocator, max_buffers,
           V4L2_MEMORY_DMABUF);
 
       /* There is no rational to not get what we asked */
@@ -905,7 +909,7 @@ no_buffers:
   {
     GST_ERROR_OBJECT (pool,
         "we received %d buffer from device '%s', we want at least %d",
-        min_buffers, obj->videodev, GST_V4L2_MIN_BUFFERS);
+        min_buffers, obj->videodev, GST_V4L2_MIN_BUFFERS (obj));
     gst_structure_free (config);
     return FALSE;
   }
@@ -932,6 +936,22 @@ cannot_import:
   }
 }
 
+static gboolean
+gst_v4l2_buffer_pool_vallocator_stop (GstV4l2BufferPool * pool)
+{
+  GstV4l2Return vret;
+
+  if (!pool->vallocator)
+    return TRUE;
+
+  vret = gst_v4l2_allocator_stop (pool->vallocator);
+
+  if (vret == GST_V4L2_BUSY)
+    GST_WARNING_OBJECT (pool, "some buffers are still outstanding");
+
+  return (vret == GST_V4L2_OK);
+}
+
 static gboolean
 gst_v4l2_buffer_pool_stop (GstBufferPool * bpool)
 {
@@ -939,7 +959,7 @@ gst_v4l2_buffer_pool_stop (GstBufferPool * bpool)
   gboolean ret;
 
   if (pool->orphaned)
-    return TRUE;
+    return gst_v4l2_buffer_pool_vallocator_stop (pool);
 
   GST_DEBUG_OBJECT (pool, "stopping pool");
 
@@ -959,16 +979,8 @@ gst_v4l2_buffer_pool_stop (GstBufferPool * bpool)
 
   ret = GST_BUFFER_POOL_CLASS (parent_class)->stop (bpool);
 
-  if (ret && pool->vallocator) {
-    GstV4l2Return vret;
-
-    vret = gst_v4l2_allocator_stop (pool->vallocator);
-
-    if (vret == GST_V4L2_BUSY)
-      GST_WARNING_OBJECT (pool, "some buffers are still outstanding");
-
-    ret = (vret == GST_V4L2_OK);
-  }
+  if (ret)
+    ret = gst_v4l2_buffer_pool_vallocator_stop (pool);
 
   return ret;
 }
@@ -995,11 +1007,16 @@ gst_v4l2_buffer_pool_orphan (GstBufferPool ** bpool)
    */
   ret = gst_v4l2_buffer_pool_stop (*bpool);
   if (!ret)
-    gst_v4l2_allocator_orphan (pool->vallocator);
+    ret = gst_v4l2_allocator_orphan (pool->vallocator);
+
+  if (!ret)
+    goto orphan_failed;
 
   pool->orphaned = TRUE;
   gst_object_unref (*bpool);
   *bpool = NULL;
+
+orphan_failed:
   return ret;
 }
 
@@ -1133,22 +1150,12 @@ gst_v4l2_buffer_pool_qbuf (GstV4l2BufferPool * pool, GstBuffer * buf,
   if (V4L2_TYPE_IS_OUTPUT (obj->type)) {
     enum v4l2_field field;
 
-    /* Except when field is set to alternate, buffer field is the same as
-     * the one defined in format */
+    /* Buffer field is the same as the one defined in format */
     if (V4L2_TYPE_IS_MULTIPLANAR (obj->type))
       field = obj->format.fmt.pix_mp.field;
     else
       field = obj->format.fmt.pix.field;
 
-    /* NB: At this moment, we can't have alternate mode because it not handled
-     * yet */
-    if (field == V4L2_FIELD_ALTERNATE) {
-      if (GST_BUFFER_FLAG_IS_SET (buf, GST_VIDEO_FRAME_FLAG_TFF))
-        field = V4L2_FIELD_TOP;
-      else
-        field = V4L2_FIELD_BOTTOM;
-    }
-
     group->buffer.field = field;
   }
 
@@ -1287,6 +1294,14 @@ gst_v4l2_buffer_pool_dqbuf (GstV4l2BufferPool * pool, GstBuffer ** buffer,
       GST_BUFFER_FLAG_UNSET (outbuf, GST_VIDEO_BUFFER_FLAG_INTERLACED);
       GST_BUFFER_FLAG_UNSET (outbuf, GST_VIDEO_BUFFER_FLAG_TFF);
       break;
+    case V4L2_FIELD_TOP:
+      GST_BUFFER_FLAG_SET (outbuf, GST_VIDEO_BUFFER_FLAG_INTERLACED);
+      GST_BUFFER_FLAG_SET (outbuf, GST_VIDEO_BUFFER_FLAG_TOP_FIELD);
+      break;
+    case V4L2_FIELD_BOTTOM:
+      GST_BUFFER_FLAG_SET (outbuf, GST_VIDEO_BUFFER_FLAG_INTERLACED);
+      GST_BUFFER_FLAG_SET (outbuf, GST_VIDEO_BUFFER_FLAG_BOTTOM_FIELD);
+      break;
     case V4L2_FIELD_INTERLACED_TB:
       GST_BUFFER_FLAG_SET (outbuf, GST_VIDEO_BUFFER_FLAG_INTERLACED);
       GST_BUFFER_FLAG_SET (outbuf, GST_VIDEO_BUFFER_FLAG_TFF);
@@ -1447,6 +1462,14 @@ gst_v4l2_buffer_pool_release_buffer (GstBufferPool * bpool, GstBuffer * buffer)
 
   GST_DEBUG_OBJECT (pool, "release buffer %p", buffer);
 
+  /* If the buffer's pool has been orphaned, dispose of it so that
+   * the pool resources can be freed */
+  if (pool->orphaned) {
+    GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_TAG_MEMORY);
+    pclass->release_buffer (bpool, buffer);
+    return;
+  }
+
   switch (obj->type) {
     case V4L2_BUF_TYPE_VIDEO_CAPTURE:
     case V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE:
@@ -1475,7 +1498,7 @@ gst_v4l2_buffer_pool_release_buffer (GstBufferPool * bpool, GstBuffer * buffer)
                 gst_v4l2_buffer_pool_qbuf (pool, buffer, group) != GST_FLOW_OK)
               pclass->release_buffer (bpool, buffer);
           } else {
-            /* Simply release invalide/modified buffer, the allocator will
+            /* Simply release invalid/modified buffer, the allocator will
              * give it back later */
             GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_TAG_MEMORY);
             pclass->release_buffer (bpool, buffer);
@@ -1505,7 +1528,7 @@ gst_v4l2_buffer_pool_release_buffer (GstBufferPool * bpool, GstBuffer * buffer)
           guint index;
 
           if (!gst_v4l2_is_buffer_valid (buffer, &group)) {
-            /* Simply release invalide/modified buffer, the allocator will
+            /* Simply release invalid/modified buffer, the allocator will
              * give it back later */
             GST_BUFFER_FLAG_SET (buffer, GST_BUFFER_FLAG_TAG_MEMORY);
             pclass->release_buffer (bpool, buffer);
@@ -1645,8 +1668,9 @@ gst_v4l2_buffer_pool_new (GstV4l2Object * obj, GstCaps * caps)
 
   /* setting a significant unique name */
   parent_name = gst_object_get_name (GST_OBJECT (obj->element));
-  name = g_strconcat (parent_name, ":", "pool:",
-      V4L2_TYPE_IS_OUTPUT (obj->type) ? "sink" : "src", NULL);
+  name = g_strdup_printf ("%s:pool%u:%s",
+      parent_name, obj->pool_seq++,
+      V4L2_TYPE_IS_OUTPUT (obj->type) ? "sink" : "src");
   g_free (parent_name);
 
   pool = (GstV4l2BufferPool *) g_object_new (GST_TYPE_V4L2_BUFFER_POOL,
@@ -1967,7 +1991,7 @@ gst_v4l2_buffer_pool_process (GstV4l2BufferPool * pool, GstBuffer ** buf)
               goto prepare_failed;
             }
 
-            /* retreive the group */
+            /* retrieve the group */
             gst_v4l2_is_buffer_valid (to_queue, &group);
           }
 
