@@ -42,8 +42,12 @@ typedef struct _GstV4l2ObjectClassHelper GstV4l2ObjectClassHelper;
 
 #include <gstv4l2bufferpool.h>
 
-/* size of v4l2 buffer pool in streaming case */
-#define GST_V4L2_MIN_BUFFERS 2
+/* size of v4l2 buffer pool in streaming case, obj->info needs to be valid */
+#define GST_V4L2_MIN_BUFFERS(obj) \
+    ((GST_VIDEO_INFO_INTERLACE_MODE (&obj->info) == \
+      GST_VIDEO_INTERLACE_MODE_ALTERNATE) ? \
+      /* 2x buffers needed with each field in its own buffer */ \
+      4 : 2)
 
 /* max frame width/height */
 #define GST_V4L2_MAX_SIZE (1<<15) /* 2^15 == 32768 */
@@ -168,6 +172,8 @@ struct _GstV4l2Object {
 
   /* optional pool */
   GstBufferPool *pool;
+  /* the sequence of pool to identify (for debugging) */
+  guint pool_seq;
 
   /* the video device's capabilities */
   struct v4l2_capability vcap;
