@@ -45,6 +45,7 @@ typedef struct
   gchar *device;
   GstCaps *sink_caps;
   GstCaps *src_caps;
+  const GstV4l2Codec *codec;
 } GstV4l2VideoEncCData;
 
 enum
@@ -283,7 +284,7 @@ gst_v4l2_video_enc_finish (GstVideoEncoder * encoder)
   }
 
   /* and ensure the processing thread has stopped in case another error
-   * occured. */
+   * occurred. */
   gst_v4l2_object_unlock (self->v4l2capture);
   gst_pad_stop_task (encoder->srcpad);
   GST_VIDEO_ENCODER_STREAM_LOCK (encoder);
@@ -419,8 +420,9 @@ negotiate_profile_and_level (GstCapsFeatures * features, GstStructure * s,
   GQueue profiles = G_QUEUE_INIT;
   GQueue levels = G_QUEUE_INIT;
   gboolean failed = FALSE;
+  const GstV4l2Codec *codec = klass->codec;
 
-  if (klass->profile_cid && get_string_list (s, "profile", &profiles)) {
+  if (codec->profile_cid && get_string_list (s, "profile", &profiles)) {
     GList *l;
 
     for (l = profiles.head; l; l = l->next) {
@@ -430,8 +432,9 @@ negotiate_profile_and_level (GstCapsFeatures * features, GstStructure * s,
 
       GST_TRACE_OBJECT (ctx->self, "Trying profile %s", profile);
 
-      control.id = klass->profile_cid;
-      control.value = v4l2_profile = klass->profile_from_string (profile);
+      control.id = codec->profile_cid;
+
+      control.value = v4l2_profile = codec->profile_from_string (profile);
 
       if (control.value < 0)
         continue;
@@ -442,7 +445,7 @@ negotiate_profile_and_level (GstCapsFeatures * features, GstStructure * s,
         break;
       }
 
-      profile = klass->profile_to_string (control.value);
+      profile = codec->profile_to_string (control.value);
 
       if (control.value == v4l2_profile) {
         ctx->profile = profile;
@@ -462,7 +465,7 @@ negotiate_profile_and_level (GstCapsFeatures * features, GstStructure * s,
     g_queue_clear (&profiles);
   }
 
-  if (!failed && klass->level_cid && get_string_list (s, "level", &levels)) {
+  if (!failed && codec->level_cid && get_string_list (s, "level", &levels)) {
     GList *l;
 
     for (l = levels.head; l; l = l->next) {
@@ -472,8 +475,8 @@ negotiate_profile_and_level (GstCapsFeatures * features, GstStructure * s,
 
       GST_TRACE_OBJECT (ctx->self, "Trying level %s", level);
 
-      control.id = klass->level_cid;
-      control.value = v4l2_level = klass->level_from_string (level);
+      control.id = codec->level_cid;
+      control.value = v4l2_level = codec->level_from_string (level);
 
       if (control.value < 0)
         continue;
@@ -484,7 +487,7 @@ negotiate_profile_and_level (GstCapsFeatures * features, GstStructure * s,
         break;
       }
 
-      level = klass->level_to_string (control.value);
+      level = codec->level_to_string (control.value);
 
       if (control.value == v4l2_level) {
         ctx->level = level;
@@ -518,6 +521,7 @@ gst_v4l2_video_enc_negotiate (GstVideoEncoder * encoder)
   struct ProfileLevelCtx ctx = { self, NULL, NULL };
   GstVideoCodecState *state;
   GstStructure *s;
+  const GstV4l2Codec *codec = klass->codec;
 
   GST_DEBUG_OBJECT (self, "Negotiating %s profile and level.",
       klass->codec_name);
@@ -543,26 +547,26 @@ gst_v4l2_video_enc_negotiate (GstVideoEncoder * encoder)
     }
   }
 
-  if (klass->profile_cid && !ctx.profile) {
+  if (codec->profile_cid && !ctx.profile) {
     struct v4l2_control control = { 0, };
 
-    control.id = klass->profile_cid;
+    control.id = codec->profile_cid;
 
     if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_G_CTRL, &control) < 0)
       goto g_ctrl_failed;
 
-    ctx.profile = klass->profile_to_string (control.value);
+    ctx.profile = codec->profile_to_string (control.value);
   }
 
-  if (klass->level_cid && !ctx.level) {
+  if (codec->level_cid && !ctx.level) {
     struct v4l2_control control = { 0, };
 
-    control.id = klass->level_cid;
+    control.id = codec->level_cid;
 
     if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_G_CTRL, &control) < 0)
       goto g_ctrl_failed;
 
-    ctx.level = klass->level_to_string (control.value);
+    ctx.level = codec->level_to_string (control.value);
   }
 
   GST_DEBUG_OBJECT (self, "Selected %s profile %s at level %s",
@@ -571,10 +575,10 @@ gst_v4l2_video_enc_negotiate (GstVideoEncoder * encoder)
   state = gst_video_encoder_get_output_state (encoder);
   s = gst_caps_get_structure (state->caps, 0);
 
-  if (klass->profile_cid)
+  if (codec->profile_cid)
     gst_structure_set (s, "profile", G_TYPE_STRING, ctx.profile, NULL);
 
-  if (klass->level_cid)
+  if (codec->level_cid)
     gst_structure_set (s, "level", G_TYPE_STRING, ctx.level, NULL);
 
   if (!GST_VIDEO_ENCODER_CLASS (parent_class)->negotiate (encoder))
@@ -720,7 +724,7 @@ gst_v4l2_video_enc_handle_frame (GstVideoEncoder * encoder,
   if (G_UNLIKELY (!g_atomic_int_get (&self->active)))
     goto flushing;
 
-  task_state = gst_pad_get_task_state (GST_VIDEO_DECODER_SRC_PAD (self));
+  task_state = gst_pad_get_task_state (GST_VIDEO_ENCODER_SRC_PAD (self));
   if (task_state == GST_TASK_STOPPED || task_state == GST_TASK_PAUSED) {
     GstBufferPool *pool = GST_BUFFER_POOL (self->v4l2output->pool);
 
@@ -735,7 +739,8 @@ gst_v4l2_video_enc_handle_frame (GstVideoEncoder * encoder,
     /* Ensure input internal pool is active */
     if (!gst_buffer_pool_is_active (pool)) {
       GstStructure *config = gst_buffer_pool_get_config (pool);
-      guint min = MAX (self->v4l2output->min_buffers, GST_V4L2_MIN_BUFFERS);
+      guint min = MAX (self->v4l2output->min_buffers,
+          GST_V4L2_MIN_BUFFERS (self->v4l2output));
 
       gst_buffer_pool_config_set_params (config, self->input_state->caps,
           self->v4l2output->info.size, min, min);
@@ -772,8 +777,7 @@ gst_v4l2_video_enc_handle_frame (GstVideoEncoder * encoder,
     GST_VIDEO_ENCODER_STREAM_LOCK (encoder);
 
     if (ret == GST_FLOW_FLUSHING) {
-      if (gst_pad_get_task_state (GST_VIDEO_DECODER_SRC_PAD (self)) !=
-          GST_TASK_STARTED)
+      if (gst_pad_get_task_state (encoder->srcpad) != GST_TASK_STARTED)
         ret = self->output_flow;
       goto drop;
     } else if (ret != GST_FLOW_OK) {
@@ -852,7 +856,7 @@ gst_v4l2_video_enc_decide_allocation (GstVideoEncoder *
   }
 
   /* FIXME This may not be entirely correct, as encoder may keep some
-   * observation withouth delaying the encoding. Linux Media API need some
+   * observation without delaying the encoding. Linux Media API need some
    * more work to explicitly expressed the decoder / encoder latency. This
    * value will then become max latency, and the reported driver latency would
    * become the min latency. */
@@ -973,8 +977,9 @@ gst_v4l2_video_enc_sink_event (GstVideoEncoder * encoder, GstEvent * event)
 {
   GstV4l2VideoEnc *self = GST_V4L2_VIDEO_ENC (encoder);
   gboolean ret;
+  GstEventType type = GST_EVENT_TYPE (event);
 
-  switch (GST_EVENT_TYPE (event)) {
+  switch (type) {
     case GST_EVENT_FLUSH_START:
       GST_DEBUG_OBJECT (self, "flush start");
       gst_v4l2_object_unlock (self->v4l2output);
@@ -986,7 +991,7 @@ gst_v4l2_video_enc_sink_event (GstVideoEncoder * encoder, GstEvent * event)
 
   ret = GST_VIDEO_ENCODER_CLASS (parent_class)->sink_event (encoder, event);
 
-  switch (GST_EVENT_TYPE (event)) {
+  switch (type) {
     case GST_EVENT_FLUSH_START:
       gst_pad_stop_task (encoder->srcpad);
       GST_DEBUG_OBJECT (self, "flush start done");
@@ -1121,8 +1126,8 @@ gst_v4l2_video_enc_subclass_init (gpointer g_class, gpointer data)
   GstV4l2VideoEncCData *cdata = data;
 
   klass->default_device = cdata->device;
+  klass->codec = cdata->codec;
 
-  /* Note: gst_pad_template_new() take the floating ref from the caps */
   gst_element_class_add_pad_template (element_class,
       gst_pad_template_new ("sink", GST_PAD_SINK, GST_PAD_ALWAYS,
           cdata->sink_caps));
@@ -1159,8 +1164,9 @@ gst_v4l2_is_video_enc (GstCaps * sink_caps, GstCaps * src_caps,
 
 void
 gst_v4l2_video_enc_register (GstPlugin * plugin, GType type,
-    const char *codec, const gchar * basename, const gchar * device_path,
-    GstCaps * sink_caps, GstCaps * codec_caps, GstCaps * src_caps)
+    const char *codec_name, const gchar * basename, const gchar * device_path,
+    const GstV4l2Codec * codec, gint video_fd, GstCaps * sink_caps,
+    GstCaps * codec_caps, GstCaps * src_caps)
 {
   GstCaps *filtered_caps;
   GTypeQuery type_query;
@@ -1168,13 +1174,27 @@ gst_v4l2_video_enc_register (GstPlugin * plugin, GType type,
   GType subtype;
   gchar *type_name;
   GstV4l2VideoEncCData *cdata;
+  GValue value = G_VALUE_INIT;
 
   filtered_caps = gst_caps_intersect (src_caps, codec_caps);
 
+  if (codec != NULL && video_fd != -1) {
+    if (gst_v4l2_codec_probe_levels (codec, video_fd, &value)) {
+      gst_caps_set_value (filtered_caps, "level", &value);
+      g_value_unset (&value);
+    }
+
+    if (gst_v4l2_codec_probe_profiles (codec, video_fd, &value)) {
+      gst_caps_set_value (filtered_caps, "profile", &value);
+      g_value_unset (&value);
+    }
+  }
+
   cdata = g_new0 (GstV4l2VideoEncCData, 1);
   cdata->device = g_strdup (device_path);
   cdata->sink_caps = gst_caps_ref (sink_caps);
   cdata->src_caps = gst_caps_ref (filtered_caps);
+  cdata->codec = codec;
 
   g_type_query (type, &type_query);
   memset (&type_info, 0, sizeof (type_info));
@@ -1188,11 +1208,11 @@ gst_v4l2_video_enc_register (GstPlugin * plugin, GType type,
    * v4l2h264enc, for any additional encoders, we create unique names. Encoder
    * names may change between boots, so this should help gain stable names for
    * the most common use cases. */
-  type_name = g_strdup_printf ("v4l2%senc", codec);
+  type_name = g_strdup_printf ("v4l2%senc", codec_name);
 
   if (g_type_from_name (type_name) != 0) {
     g_free (type_name);
-    type_name = g_strdup_printf ("v4l2%s%senc", basename, codec);
+    type_name = g_strdup_printf ("v4l2%s%senc", basename, codec_name);
   }
 
   subtype = g_type_register_static (type, type_name, &type_info, 0);
