@@ -27,20 +27,17 @@
 
 /**
  * SECTION:gsttunerchannel
+ * @title: GstTunerChannel
  * @short_description: A channel from an element implementing the #GstTuner
  * interface.
  *
- * <refsect2>
- * <para>The #GstTunerChannel object is provided by an element implementing
+ * The #GstTunerChannel object is provided by an element implementing
  * the #GstTuner interface.
- * </para>
- * <para>
+ *
  * GstTunerChannel provides a name and flags to determine the type and
  * capabilities of the channel. If the GST_TUNER_CHANNEL_FREQUENCY flag is
  * set, then the channel also information about the minimum and maximum
  * frequency, and range of the reported signal strength.
- * </para>
- * </refsect2>
  */
 
 enum
@@ -51,47 +48,17 @@ enum
   LAST_SIGNAL
 };
 
-static void gst_tuner_channel_class_init (GstTunerChannelClass * klass);
-static void gst_tuner_channel_init (GstTunerChannel * channel);
+G_DEFINE_TYPE (GstTunerChannel, gst_tuner_channel, G_TYPE_OBJECT);
+
 static void gst_tuner_channel_dispose (GObject * object);
 
-static GObjectClass *parent_class = NULL;
 static guint signals[LAST_SIGNAL] = { 0 };
 
-GType
-gst_tuner_channel_get_type (void)
-{
-  static GType gst_tuner_channel_type = 0;
-
-  if (!gst_tuner_channel_type) {
-    static const GTypeInfo tuner_channel_info = {
-      sizeof (GstTunerChannelClass),
-      NULL,
-      NULL,
-      (GClassInitFunc) gst_tuner_channel_class_init,
-      NULL,
-      NULL,
-      sizeof (GstTunerChannel),
-      0,
-      (GInstanceInitFunc) gst_tuner_channel_init,
-      NULL
-    };
-
-    gst_tuner_channel_type =
-        g_type_register_static (G_TYPE_OBJECT,
-        "GstTunerChannel", &tuner_channel_info, 0);
-  }
-
-  return gst_tuner_channel_type;
-}
-
 static void
 gst_tuner_channel_class_init (GstTunerChannelClass * klass)
 {
   GObjectClass *object_klass = (GObjectClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   /**
    * GstTunerChannel::frequency-changed:
    * @tunerchannel: The #GstTunerChannel
@@ -102,9 +69,8 @@ gst_tuner_channel_class_init (GstTunerChannelClass * klass)
   signals[SIGNAL_FREQUENCY_CHANGED] =
       g_signal_new ("frequency-changed", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST,
-      G_STRUCT_OFFSET (GstTunerChannelClass,
-          frequency_changed),
-      NULL, NULL, g_cclosure_marshal_VOID__ULONG, G_TYPE_NONE, 1, G_TYPE_ULONG);
+      G_STRUCT_OFFSET (GstTunerChannelClass, frequency_changed),
+      NULL, NULL, NULL, G_TYPE_NONE, 1, G_TYPE_ULONG);
   /**
    * GstTunerChannel::signal-changed:
    * @tunerchannel: The #GstTunerChannel
@@ -117,9 +83,8 @@ gst_tuner_channel_class_init (GstTunerChannelClass * klass)
   signals[SIGNAL_SIGNAL_CHANGED] =
       g_signal_new ("signal-changed", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST,
-      G_STRUCT_OFFSET (GstTunerChannelClass,
-          signal_changed),
-      NULL, NULL, g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
+      G_STRUCT_OFFSET (GstTunerChannelClass, signal_changed),
+      NULL, NULL, NULL, G_TYPE_NONE, 1, G_TYPE_INT);
 
   object_klass->dispose = gst_tuner_channel_dispose;
 }
@@ -143,6 +108,5 @@ gst_tuner_channel_dispose (GObject * object)
     channel->label = NULL;
   }
 
-  if (parent_class->dispose)
-    parent_class->dispose (object);
+  G_OBJECT_CLASS (gst_tuner_channel_parent_class)->dispose (object);
 }
